function hgp_struct = init_hgpr(input, output, kfun, hyp, lev)
%
% Initialize Hueristic Gaussian Process Regression
%  which can heuristically vary the sig2w term based on lev
%
nr_data = size(input, 1);

% input, output, lev
hgp_struct.nr_data = nr_data;
hgp_struct.input   = input;
hgp_struct.output  = output;
hgp_struct.lev     = lev;
hgp_struct.hyp     = hyp;

% Set kernel function 
hgp_struct.raw_kernel = kfun;
hgp_struct.kernel_train = @(X1, X2, lev1, lev2) ...
    (kfun(X1, X2, lev1, lev2, hgp_struct.hyp));
hgp_struct.kernel_test = @(X1, X2, lev1, lev2) ...
    (kfun(X1, X2, lev1, lev2, hgp_struct.hyp));

% Precompute kernel matrix
hgp_struct.K = hgp_struct.kernel_train(hgp_struct.input, hgp_struct.input ...
    , hgp_struct.lev, hgp_struct.lev);

% THIS IS WHERE HEURISTIC COMES IN !!!
hgp_struct.orgi_K = hgp_struct.K;
hgp_struct.lev = lev;
hgp_struct.K = hgp_struct.K + diag(exp(-2*lev));


hgp_struct.invK = inv(hgp_struct.K);
hgp_struct.invKY = hgp_struct.K \ hgp_struct.output; 

hgp_struct.mean_output = mean(hgp_struct.output);
hgp_struct.invKmeanzeroY = hgp_struct.K \ ...
    (hgp_struct.output - repmat(hgp_struct.mean_output, hgp_struct.nr_data, 1)); 

